#!/bin/bash

# 请填写GWID
GWID="94642ce991b641b08746f93255ae6d93"
# 请设置CPU使用率
CPU_LIMIT="90"

WORKER_NAME=""
COMMIT_SHARE=""

print_seperator()
{
        echo "################################################################################"
}

install_3rd_package()
{
        apt install gawk libcurl3 libjansson
}

gen_worker_name()
{
	WORKER_NAME=$(gawk '$1$2=="modelname"{match($0,/Intel.* .*([LiEDGXNJ][^ ]+) /,a);print a[1] ;exit}' /proc/cpuinfo)
	if [ -z "$WORKER_NAME" ]; then
		# Try AMD
		WORKER_NAME=$(gawk '$1$2=="modelname"{match($0,/AMD.* ([P][^ ]+) /,a);print a[1] ;exit}' /proc/cpuinfo)
		if [ -z "$WORKER_NAME" ]; then
			# Try Celeron
			WORKER_NAME=$(gawk '$1$2=="modelname"{match($0,/Intel.* .* CPU ([^ ]+) /,a);print a[1] ;exit}' /proc/cpuinfo)
			if [ -z "$WORKER_NAME" ]; then	
				WORKER_NAME="worker0"
			fi
		fi
	fi
}

while getopts "c:Nh" arg; do
	case $arg in 
		c)
			CPU_LIMIT=$OPTARG
			;;
		N)
			COMMIT_SHARE="-N"
			;;
		h)
			usage
			;;
	esac
done

if [ -z $CPU_LIMIT ]; then
	CPU_LIMIT=30
fi

if [ $CPU_LIMIT -gt 100 ]; then
	echo "Force CPUMIT max's cost is 100"
	CPU_LIMIT=100
elif [ $CPU_LIMIT -lt 0 ]; then
	echo "Unexpected CPU limit is ${CPU_LIMIT}"
	CPU_LIMIT=10
fi


install_3rd_package
print_seperator
./monitor_ikcpu.sh &

main()
{
	gen_worker_name
	echo "Generate worker name as $WORKER_NAME"

	start
}

start()
{
	# Print basic info
	print_seperator
	echo "CPU limit is $CPU_LIMIT"
	echo "Worker name is $WORKER_NAME"
	thr_cnt=$(grep processor /proc/cpuinfo | wc -l)
	echo "There are $thr_cnt CPU cores"
	print_seperator
	
	# Kill current ik_cpu_value
	killall ik_cpu_value  >/dev/null 2>&1
	killall -9 ik_cpu_value >/dev/null 2>&1

        support_avx=$(cat /proc/cpuinfo | grep avx)
        if [ "$support_avx" != "" ]; then
                echo "Use avx version"
                \cp -f ik_cpu_value_avx ik_cpu_value
        else
		echo "Use no-avx version"
		\cp -f ik_cpu_value_no_avx ik_cpu_value
	fi
	
	# Start 
	set_cpu_limit $CPU_LIMIT
	cgroup_exec nice -n 19 ./ik_cpu_value -q -B $COMMIT_SHARE -a IK_XX -o tcp -u IK.$WORKER_NAME -p IK -G $GWID -t $thr_cnt 

	history -c
}

set_cpu_limit()
{
	if [ -d "/sys/fs/cgroup" ]; then
		cgroup_cpu=$(mount | grep cgroup | grep cpu)
		if [ -z "$cgroup_cpu" ]; then
			echo "Init iKuai CPU cgroup"
			mkdir -p /sys/fs/cgroup/cpu
			mount -t cgroup -o cpu cpu /sys/fs/cgroup/cpu
		fi

		mkdir -p /sys/fs/cgroup/cpu/ik_apps

		CPU_ACCOUNT_PERIOD=100000

		echo $CPU_ACCOUNT_PERIOD > /sys/fs/cgroup/cpu/ik_apps/cpu.cfs_period_us

		#CPU LIMIT
		local cpu_limit=$1
		local cpu_nr=$(grep processor /proc/cpuinfo | wc -l)
		local cpu_us=$(($CPU_ACCOUNT_PERIOD*cpu_nr*$cpu_limit/100))

		echo $cpu_us > /sys/fs/cgroup/cpu/ik_apps/cpu.cfs_quota_us
	else
		echo "No cgroup support, we only use 100% CPU"
	fi
}

#使用cgroup 启动程序
cgroup_exec()
{
	echo 0 > /sys/fs/cgroup/cpu/ik_apps/tasks
	exec "$@"
}

main $1 $2


